import type { UnsubscribeFn } from '@closeio/rn-common/dist/src/Types';
import type { BridgeMessageDefinition, BridgeMessageParcel, BridgeMessagePayload, BridgeMessageResponse, BridgeOptions, BridgeOptionsMaybe, BridgeOptionsNone, BridgeReceivables, BridgeSendables } from './Types';
import { BridgeQueue } from './BridgeQueue';
type BridgeConnector<Options extends BridgeOptions = BridgeOptionsNone> = {
    sendMessage: (parcel: BridgeMessageParcel, ...options: BridgeOptionsMaybe<Options>) => void;
    onReceiveMessage: (action: string, callback: (parcel: BridgeMessageParcel) => Promise<void>, ...options: BridgeOptionsMaybe<Options>) => UnsubscribeFn;
};
export declare class Bridge<Messages extends BridgeMessageDefinition[], Options extends BridgeOptions = BridgeOptionsNone> {
    static readonly RESPONSE_ACTION_PREFIX = "_response_";
    static isResponseAction(action?: string): boolean | undefined;
    readonly responseTimeoutMs: number;
    readonly connector: BridgeConnector<Options>;
    readonly queue: BridgeQueue;
    readonly waitingFor: Record<string, string>;
    constructor(connector: BridgeConnector<Options>, params?: {
        canSendFn?: () => boolean;
        responseTimeoutMs?: number;
    });
    readonly send: <Action extends keyof BridgeSendables<Messages>>(action: Action & string, payload: BridgeMessagePayload<BridgeSendables<Messages>[Action]>, ...options: BridgeOptionsMaybe<Options>) => void;
    readonly sendWithResult: <Action extends keyof BridgeSendables<Messages>>(action: Action & string, payload: BridgeMessagePayload<BridgeSendables<Messages>[Action]>, ...options: BridgeOptionsMaybe<Options>) => Promise<BridgeMessageResponse<BridgeSendables<Messages>[Action]>>;
    private readonly sendWithoutQueue;
    readonly onReceive: <Action extends keyof BridgeReceivables<Messages>>(action: Action & string, callback: (payload: BridgeMessagePayload<BridgeReceivables<Messages>[Action]>) => Promise<BridgeMessageResponse<BridgeReceivables<Messages>[Action]>>, ...options: BridgeOptionsMaybe<Options>) => UnsubscribeFn;
}
export {};
