import type { Util_FlatType } from '@closeio/rn-common/dist/src/Types';
export type BridgeMessageParcel = {
    action: string;
    payload: object;
    __responseAction?: string;
};
export type BridgeOptionsNone = '_none';
export type BridgeOptions = object | BridgeOptionsNone;
export type BridgeOptionsMaybe<Options extends BridgeOptions> = Options extends '_none' ? [] : [options: Options];
export type BridgeMessageShapeDefinition<Payload extends object = {}, Response extends object | void = void> = {
    payload: Payload;
    response: Response;
};
type BridgeMessageShape<Action extends string, Payload extends object = {}, Response extends object | void = void> = {
    action: Action;
    payload: Payload;
    response: Response;
};
export type BridgeMessageDefinition<Action extends string = string, Sendable extends BridgeMessageShapeDefinition<any, any> | undefined = undefined, Receivable extends BridgeMessageShapeDefinition<any, any> | undefined = undefined> = {
    action: Action;
} & (Sendable extends undefined ? {} : {
    sendable: Sendable;
}) & (Receivable extends undefined ? {} : {
    receivable: Receivable;
});
export type FlipBridgeMessageDefinition<BMD> = BMD extends BridgeMessageDefinition<infer Action, infer Sendable, infer Receivable> ? {
    action: Action;
} & (Sendable extends BridgeMessageShapeDefinition<infer Payload, infer Response> ? {
    receivable: {
        payload: Payload;
        response: Response;
    };
} : {}) & (Receivable extends BridgeMessageShapeDefinition<infer Payload, infer Response> ? {
    sendable: {
        payload: Payload;
        response: Response;
    };
} : {}) : never;
type ExtractMessageShapeByDestination<Dest extends 'sendable' | 'receivable', First> = First extends {
    action: infer Name;
    sendable?: infer Sendable;
    receivable?: infer Receivable;
} ? (Dest extends 'sendable' ? Sendable extends BridgeMessageShapeDefinition<any, any> ? {
    [Action in Name & string]: {
        action: Action;
        payload: Sendable extends BridgeMessageShapeDefinition<infer Payload, any> ? Payload : void;
        response: Sendable extends BridgeMessageShapeDefinition<any, infer Response> ? Response : void;
    };
} : {} : Receivable extends BridgeMessageShapeDefinition<any, any> ? {
    [Action in Name & string]: {
        action: Action;
        payload: Receivable extends BridgeMessageShapeDefinition<infer Payload, any> ? Payload : void;
        response: Receivable extends BridgeMessageShapeDefinition<any, infer Response> ? Response : void;
    };
} : {}) : {};
export type BridgeMessagesByDestination<Dest extends 'sendable' | 'receivable', Messages extends BridgeMessageDefinition[]> = Util_FlatType<Messages extends [infer First, ...infer Rest] ? First extends BridgeMessageDefinition ? ExtractMessageShapeByDestination<Dest, First> & (Rest extends BridgeMessageDefinition[] ? BridgeMessagesByDestination<Dest, Rest> : {}) : {} : {}>;
export type FlipBridge<Messages extends BridgeMessageDefinition[]> = Util_FlatType<Messages extends [infer First, ...infer Rest] ? [
    FlipBridgeMessageDefinition<First>,
    ...(Rest extends BridgeMessageDefinition[] ? FlipBridge<Rest> : [])
] : []>;
export type BridgeSendables<Messages extends BridgeMessageDefinition[]> = BridgeMessagesByDestination<'sendable', Messages>;
export type BridgeReceivables<Messages extends BridgeMessageDefinition[]> = BridgeMessagesByDestination<'receivable', Messages>;
export type BridgeMessagePayload<Message> = Message extends BridgeMessageShape<any, infer Payload, any> ? Payload : never;
export type BridgeMessageResponse<Message> = Message extends BridgeMessageShape<any, any, infer Response> ? Response : never;
export {};
